

import logging

from typing import List
from typing import Optional
from pydantic import Field
from pydantic import BaseModel
from fastapi import Body
from fastapi import Depends
from fastapi import APIRouter
from fastapi import HTTPException

from wpasupplicantconf import WpaSupplicantConf

from ebs.linuxnode.netconfig import config
from ebs.linuxnode.netconfig.core import app
from ebs.linuxnode.netconfig.core import auth_token
from ebs.linuxnode.netconfig.core import ActionResultModel

logger = logging.getLogger(__name__)
WPA_SUPPLICANT_PATH = config.wpa_supplicant_path

wifi_router = APIRouter(prefix='/wifi',
                        dependencies=[Depends(auth_token)])


class WiFiScanProxy(object):
    pass


class WifiNetworkModel(BaseModel):
    ssid: str = Field(None, title="Wifi SSID")
    psk: str = Field(None, title="Wifi PSK (WPA2)")


class WPASupplicantProxy(object):
    def __init__(self, cpath='/etc/wpa_supplicant/wpa_supplicant.conf'):
        self._cpath = cpath
        self._config = self._read_config()

    def _read_config(self):
        logger.info("Reading WPA supplicant configuration from {}".format(self._cpath))
        with open(self._cpath, 'r') as f:
            lines = f.readlines()
            return WpaSupplicantConf(lines)

    def _write_config(self):
        logger.info("Writing WPA supplicant configuration to {}".format(self._cpath))
        with open(self._cpath, 'w') as f:
            self._config.write(f)

    def show_networks(self):
        networks = self._config.networks()
        return [WifiNetworkModel(ssid=k, psk=v['psk'])
                for k, v in networks.items()]

    def has_network(self, ssid):
        return ssid in self._config.networks()

    def add_network(self, ssid, psk, **kwargs):
        logger.info("Adding WiFi network '{}' with psk '{}'".format(ssid, psk))
        self._config.add_network(ssid, psk=psk, key_mgmt="WPA-PSK", **kwargs)
        self._write_config()

    def remove_network(self, ssid):
        logger.info("Removing WiFi network '{}'".format(ssid))
        self._config.remove_network(ssid)
        self._write_config()


_wpa_supplicant: Optional[WPASupplicantProxy] = None


@app.on_event('startup')
async def init():
    global _wpa_supplicant
    _wpa_supplicant = WPASupplicantProxy(WPA_SUPPLICANT_PATH)


@wifi_router.get("/networks/show", response_model=List[WifiNetworkModel], status_code=200)
async def show_configured_wifi_networks():
    networks = _wpa_supplicant.show_networks()
    logger.info("Currently configured networks :\n{}".format(
        "\n".join(["{:20} {}".format(x.ssid, x.psk)
                   for x in networks])))
    return networks


@wifi_router.post("/networks/add", response_model=ActionResultModel, status_code=201)
async def add_wifi_network(network: WifiNetworkModel):
    if _wpa_supplicant.has_network(network.ssid):
        raise HTTPException(
            status_code=409,
            detail="SSID '{}' already exists. Modify the existing network or remove it and try again."
                   "".format(network.ssid)
        )
    _wpa_supplicant.add_network(ssid=network.ssid, psk=network.psk)
    return {"result": True}


@wifi_router.post("/networks/remove", response_model=ActionResultModel, status_code=200)
async def remove_wifi_network(ssid: str = Body(...)):
    if not _wpa_supplicant.has_network(ssid):
        raise HTTPException(
            status_code=416,
            detail="SSID '{}' not recognized. Cannot remove.".format(ssid)
        )
    _wpa_supplicant.remove_network(ssid=ssid)
    return {"result": True}


@wifi_router.get("/status")
async def wifi_network_status():
    return {"message": "WNS"}


@wifi_router.get("/scan")
async def scan_wifi_networks():
    return {"message": "SN"}





import os
import logging
import sys
from loguru import logger

from uvicorn import Config, Server
from fastapi.staticfiles import StaticFiles


LOG_LEVEL = logging.getLevelName(os.environ.get("LOG_LEVEL", "INFO"))
PORT = int(os.environ.get("NETCONFIG_PORT", "8039"))
JSON_LOGS = True if os.environ.get("JSON_LOGS", "0") == "1" else False


class InterceptHandler(logging.Handler):
    def emit(self, record):
        # Get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


def setup_logging():
    # intercept everything at the root logger
    logging.root.handlers = [InterceptHandler()]
    logging.root.setLevel(LOG_LEVEL)

    # remove every other logger's handlers
    # and propagate to root logger
    for name in logging.root.manager.loggerDict.keys():
        logging.getLogger(name).handlers = []
        logging.getLogger(name).propagate = True

    # configure loguru
    logger.configure(handlers=[{"sink": sys.stdout, "serialize": JSON_LOGS}])
    logger.add("/var/log/ebs/netconfig.log", level="INFO",
               rotation="1 week", retention="14 days")


from ebs.linuxnode.netconfig.core import api_app
from ebs.linuxnode.netconfig.core import app


class SPAStaticFiles(StaticFiles):
    async def get_response(self, path: str, scope):
        response = await super().get_response(path, scope)
        if response.status_code == 404:
            response = await super().get_response('.', scope)
        return response


def build_server():
    from ebs.linuxnode.netconfig.core import auth_router
    api_app.include_router(auth_router)
    from ebs.linuxnode.netconfig.wifi import wifi_router
    api_app.include_router(wifi_router)
    spa_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "app")
    app.mount('/api', api_app)
    app.mount('/', StaticFiles(directory=spa_path, html=True), name="spa_app")


build_server()


def run_server():
    server = Server(
        Config(
            "ebs.linuxnode.netconfig.server:app",
            host="0.0.0.0",
            port=PORT,
            log_level=LOG_LEVEL,
        ),
    )
    # setup logging last, to make sure no library overwrites it
    # (they shouldn't, but it happens)
    setup_logging()
    server.run()


if __name__ == '__main__':
    run_server()



import setuptools

_requires = [
    'six',
    'appdirs',
    'setuptools-scm',
    'fastapi',
    'python-multipart',
    'python-jose[cryptography]',
    'passlib[bcrypt]',
    'aiofiles',
    'uvicorn',
    'loguru',
    'wpasupplicantconf',
]

setuptools.setup(
    name='ebs-linuxnode-netconfig',
    url='',

    author='Chintalagiri Shashank',
    author_email='shashank.chintalagiri@gmail.com',

    description='Network configuration infrastructure for embedded linux nodes',
    long_description='',

    packages=setuptools.find_packages(),
    package_dir={'ebs.linuxnode.netconfig': 'ebs/linuxnode/netconfig'},
    package_data={'ebs.linuxnode.netconfig': ['app/*.*',
                                              'app/static/css/*.*',
                                              'app/static/js/*.*',
                                              'app/static/media/*.*',
                                              'default/config.ini']},

    install_requires=_requires,
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX :: Linux',
    ],
    entry_points={
          'console_scripts': [
              'netconfig = ebs.linuxnode.netconfig.server:run_server'
          ]
    },
)





import os
import sys
import shutil
import pkg_resources
from six.moves.configparser import ConfigParser
from appdirs import user_config_dir
from secrets import token_hex


class NetconfigConfig(object):
    _appname = os.path.join('ebs', 'netconfig')
    _root = os.path.abspath(os.path.dirname(__file__))
    _roots = [_root]
    _config_file = os.path.join(user_config_dir(_appname), 'config.ini')

    def __init__(self):
        if not os.path.exists(os.path.dirname(self._config_file)):
            os.makedirs(os.path.dirname(self._config_file), exist_ok=True)
        if not os.path.exists(self._config_file):
            _ROOT = os.path.abspath(os.path.dirname(__file__))
            shutil.copy(os.path.join(_ROOT, 'default/config.ini'), self._config_file)
        self._config = ConfigParser()
        print("Reading Config File {}".format(self._config_file))
        self._config.read(self._config_file)
        print("EBS Linux Node NetConfig, version {0}".format(self.netconfig_version))

    @property
    def netconfig_version(self):
        return pkg_resources.get_distribution('ebs-linuxnode-netconfig').version

    def _write_config(self):
        with open(self._config_file, 'w') as configfile:
            self._config.write(configfile)

    def _check_section(self, section):
        if not self._config.has_section(section):
            self._config.add_section(section)
            self._write_config()

    @property
    def auth_secret_key(self):
        if not self._config.has_option(section='auth', option='secret_key'):
            self._config.set('auth', 'secret_key', token_hex(32))
            self._write_config()
        return self._config.get('auth', 'secret_key')

    @property
    def auth_username(self):
        return self._config.get('auth', 'username')

    @property
    def auth_password(self):
        return self._config.get('auth', 'password')

    @property
    def wifi_device(self):
        return self._config.get('wifi', 'device')

    @property
    def wpa_supplicant_path(self):
        return self._config.get('wifi', 'wpa_supplicant_path')


_config = NetconfigConfig()
sys.modules[__name__] = _config





from typing import Optional
from datetime import datetime
from datetime import timedelta

from fastapi import FastAPI
from fastapi import Depends
from fastapi import HTTPException
from fastapi import status
from fastapi import APIRouter

from fastapi.security import OAuth2PasswordBearer
from fastapi.security import OAuth2PasswordRequestForm
from fastapi.middleware.cors import CORSMiddleware


from pydantic import BaseModel
from passlib.context import CryptContext
from jose import jwt
from jose import JWTError

from ebs.linuxnode.netconfig import config

SECRET_KEY = config.auth_secret_key
AUTH_USERNAME = config.auth_username
AUTH_HASHED_PASSWORD = config.auth_password

ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 30

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
oauth2_scheme = OAuth2PasswordBearer(tokenUrl='token')

api_app = FastAPI()
app = FastAPI()

origins = ['*']

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"]
)


auth_router = APIRouter()

auth_db = {
    AUTH_USERNAME: {
        "username": AUTH_USERNAME,
        "hashed_password": AUTH_HASHED_PASSWORD,
    },
}


class ActionResultModel(BaseModel):
    result: bool


class Token(BaseModel):
    access_token: str
    token_type: str


class TokenData(BaseModel):
    username: Optional[str] = None


class UserModel(BaseModel):
    username: str
    disabled: Optional[bool] = False


class AuthModel(UserModel):
    hashed_password: str


def get_user(db, username: str):
    if username in db:
        user_dict = db[username]
        return AuthModel(**user_dict)


def verify_password(plain_password, hashed_password):
    return pwd_context.verify(plain_password, hashed_password)


def hash_password(password):
    return pwd_context.hash(password)


async def auth_token(token: str = Depends(oauth2_scheme)):
    return token


async def get_current_user(token: str = Depends(oauth2_scheme)):
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate credentials",
        headers={"WWW-Authenticate": "Bearer"},
    )
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
        token_data = TokenData(username=username)
    except JWTError:
        raise credentials_exception
    user = get_user(auth_db, username=token_data.username)
    if user is None:
        raise credentials_exception
    return user


async def get_current_active_user(current_user: UserModel = Depends(get_current_user)):
    if current_user.disabled:
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


def authenticate_user(db, username: str, password: str):
    user = get_user(db, username)
    if not user:
        return False
    if not verify_password(password, user.hashed_password):
        return False
    return user


def create_access_token(data: dict, expires_delta: Optional[timedelta] = None):
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt


@auth_router.post("/token", response_model=Token)
async def login(form_data: OAuth2PasswordRequestForm = Depends()):
    user = authenticate_user(auth_db, form_data.username, form_data.password)
    if not user:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Incorrect username or password",
            headers={"WWW-Authenticate": "Bearer"},
        )
    access_token_expires = timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = create_access_token(
        data={"sub": user.username}, expires_delta=access_token_expires
    )
    return {"access_token": access_token, "token_type": "bearer"}


@auth_router.get("/verifyToken")
async def verify_token(token: str = Depends(auth_token)):
    return token
